# sb - Serene Bach
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb;
use sb::Config ();
use sb::Language ();
use sb::Driver ();
use sb::Interface ();
use sb::Plugin ();
use sb::App ();
use sb::Object ();
use base qw( sb::RequestObject );
use vars qw( $VERSION $WEBPAGE $PRODUCT $ACRONYM $BUILDNO $MODPERL $DBTABLE );
# --------------------------------------
# // defines / static variables
$PRODUCT = 'Serene Bach';
$WEBPAGE = 'http://serenebach.net/';
$ACRONYM = 'SB3';
$BUILDNO = 'SB300_20081208';
$DBTABLE = 2;
$VERSION = '3.00b';
# 3.00b [2008/11/23] Serene Bach 3 beta - added debug / modified initialize
# 3.00b [2007/12/04] Serene Bach 3 beta
# 3.00a [2007/09/20] Serene Bach 3 alpha
sub DEFAULT_MODULE (){ 'Main' };
sub TEMPFILE       (){ 'install.tmp' };
sub DEBUG_FILE     (){ 'debug.log' };
# --------------------------------------
# // initializer
BEGIN
{
	$MODPERL = undef;
	if ($ENV{'MOD_PERL'})
	{ # -- mod_perl
		eval { require mod_perl2; };
		if (!$@)
		{ # Apache2 + mod_perl2
			require Apache2::RequestUtil;
			$MODPERL = 2;
		}
		else
		{ # Apache + mod_perl
			eval { require mod_perl; };
			die('mod_perl or mod_perl2 is required to run:' . $@) if ($@);
			require Apache;
			$MODPERL = 1;
		}
	}
}
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my $name = shift;
	my %param = (
		'without_db' => undef,
		@_
	);
	$self->SUPER::initialize(
		'name'   => $name || DEFAULT_MODULE,
		'conf'   => sb::Config->new(),
		'lang'   => undef,
		'driver' => undef,
		'plugin' => undef,
	);
	$self->generate_accessor('name','driver','conf','lang','plugin');
	return $self if ( !$self->conf );
	# initialize a configuration manager
	$self->conf->load($self->conf->config_file()); # load stored config data
	$self->conf->verify_values();
	# initialize a language handler
	my $lang = sb::Language->new($self->conf->value('language'));
	if ( $lang )
	{
		$lang->load($self->conf->lang_resource($lang->language));
		$self->lang($lang);
	}
	return $self if ( !$self->lang );
	# initailize a database driver
	if ( !$param{'without_db'} )
	{
		eval {
			my $driver = sb::Driver->new(
				$self->conf->value('driver'),
				%{$self->conf->value('db_param')}
			);
			$self->driver($driver) if ($driver);
		};
	}
	return $self;
}
sub DESTROY
{
	my $self = shift;
	$self->driver->finish if ($self->driver);
	  ($MODPERL == 2) ? ModPerl::Util::exit(0)
	: ($MODPERL == 1) ? Apache::exit(0)
	: CORE::exit(0);
}
# --------------------------------------
# // public functions - class method
sub run
{
	my $class = shift;
	my $sb = ref($class) ? $class : $class->new(@_);
	my %instance = (
		'conf'   => $sb->conf,
		'lang'   => $sb->lang,
		'cgi'    => undef,
		'apache' => undef,
	);
	$instance{'apache'} = 
		  ($MODPERL == 2) ? Apache2::RequestUtil->request
		: ($MODPERL == 1) ? Apache->request
		:                   undef;
	$instance{'cgi'} = sb::Interface->new(
		'req'    => $instance{'apache'},
		'parser' => {
			'max_data' => $sb->conf->value('max_data'),
			'decode_flag' => '__mode=bm',
		},
	);
	if ( $sb->driver )
	{ # when driver is available, we can initialize a plugin manager as well.
		my $plugin = sb::Plugin->new('dir'=>$sb->conf->value('dir_plugin'));
		if ( $plugin )
		{
			$plugin->load_plugins();
			$sb->plugin($plugin);
		}
	}
	die('failed to get sb::Config : ' . $@) if ( !$sb->conf );
	die('failed to get sb::Language : ' . $@) if (!$sb->lang );
	eval { # main routine
		my $app = 'sb::App::' . $sb->module($instance{'cgi'});
		$app = 'sb::App' if (!$app->isa('sb::App') and !load_module($app));
		$app->run(%instance);
	};
	sb::App->new(%instance)->error($@) if ($@);
}
# --------------------------------------
# // public functions
sub pid
{ # get unique id
	my $self = shift;
	my $id = undef;
	eval {
		require Apache2::MPM;
		require APR::OS;
		$id = $$ . '.' . ${ APR::OS::current_thread_id() } if (Apache2::MPM->is_threaded);
	};
	return $$ if ($@);
	return $$ if (! defined($id));
	return $id;
}
sub string
{ # wrapper function of sb::Language::string
	my $self = shift;
	return sb->get->lang->string(@_);
}
sub debug
{ # output debug message
	my $self = shift;
	my $msg = shift;
	my $lvl = sb::Config->get->value('debug_level');
	return if ( !$lvl or $lvl eq 'none' );
	if ( $msg !~ /\n/ )
	{
		my ($module, $file, $line) = caller;
		$msg .= ' at ' . $file . ' line ' . $line . "\n";
	}
	# [note] we cannot use sb::Time here because the instance of "sb" may not be ready.
	my ($se,$mi,$ho,$md,$mo,$ye,$wd,$yd,$is) = localtime();
	$msg = sprintf('%04d.%02d.%02d %02d:%02d:%02d %s',$ye + 1900,$mo + 1,$md,$ho,$mi,$se,$msg);
	if ( $lvl == 1 or $lvl eq 'warn' )
	{
		warn( $msg );
	}
	elsif ( $lvl == 2 or $lvl eq 'file' )
	{
		my $fh = undef;
		my $file = sb::Config->get->value('dir_data') . DEBUG_FILE;
		(-e $file) ? open($fh,">>$file") : open($fh,">$file");
		if ( $fh )
		{
			binmode($fh);
			print $fh $msg;
		}
		close($fh);
	}
}
sub module
{ # check module and return module name to load
	my $self = shift;
	my $cgi  = shift;
	if ($self->name ne 'Install')
	{ # check the installation
		my $file = sb::Config->get->value('dir_data') . TEMPFILE;
		die("not installed yet\n") if (-e $file);
	}
	SWITCH_NAME: {
		$_ = $self->name;
		/^Admin$/ && do {
			return 
				  ($cgi->value('__rebuild') ne '') ? 'Builder'
				: ($cgi->xmlrpc)                   ? 'Xmlrpc'
				:                                    $self->name;
		};
		/^Main/ && do {
			my $ua = sb::Config->get->value('basic_mobswitch');
			return
				  ($ua and $cgi->user_agent =~ /^$ua/) ? 'Mobile'
				: ($cgi->value('feed') ne '')          ? 'Feed'
				: ($cgi->value('rsd') eq 'on')         ? 'Rsd'
				:                                        $self->name;
		};
	} # end of SWITCH_NAME
	return $self->name;
}
sub load_module
{ # wrapper function of sb::Object::load_module
	my $module = shift;
	return &sb::Object::load_module($module);
}
1;
=head1 NAME

B<sb> - Serene Bach base module

=head1 SYNOPSIS

 use sb;
 sb->run('Main');

=head1 DESCRIPTION

B<sb> is a simple core framework especially for CGI script.  
sb consists of some fundamental classes.  
sb.pm basically works as a system loader.

=head2 sb->run( module, [without_db=>1] );

I<run> method provides booting module which subclass of sb::App.  
I<run> method is just called usually in a CGI application.  
I<run> method includes whole application process between construction and destruction.

=head2 sb->new( module, [without_db=>1] );

I<new> is a cunstructor of sb.
When sb is constructed, sb::Config, sb::Language are loaded.

"without_db" option indicates booting option for sb.  
When "without_db" is set, sb is booted without sb::Driver.

=head1 AUTHOR

Takuya Otani <takuya.otani@gmail.com> 

=head1 COPYRIGHT & LICENSE

Copyright (C) 2005 Takuya Otani@SimpleBoxes / SerendipityNZ Ltd.

=cut
__END__
